import _plotly_utils.basevalidators


class XtypeValidator(_plotly_utils.basevalidators.EnumeratedValidator):

    def __init__(self, plotly_name='xtype', parent_name='heatmapgl', **kwargs):
        super(XtypeValidator, self).__init__(
            plotly_name=plotly_name,
            parent_name=parent_name,
            edit_type='calc',
            role='info',
            values=['array', 'scaled'],
            **kwargs
        )



from ._textfont import Textfont
from ._stream import Stream
from ._projection import Projection
from plotly.graph_objs.scatter3d import projection
from ._marker import Marker
from plotly.graph_objs.scatter3d import marker
from ._line import Line
from ._hoverlabel import Hoverlabel
from plotly.graph_objs.scatter3d import hoverlabel
from ._error_z import ErrorZ
from ._error_y import ErrorY
from ._error_x import ErrorX



import _plotly_utils.basevalidators


class BoxValidator(_plotly_utils.basevalidators.CompoundValidator):

    def __init__(self, plotly_name='box', parent_name='violin', **kwargs):
        super(BoxValidator, self).__init__(
            plotly_name=plotly_name,
            parent_name=parent_name,
            data_class_str='Box',
            data_docs="""
            fillcolor
                Sets the inner box plot fill color.
            line
                plotly.graph_objs.violin.box.Line instance or
                dict with compatible properties
            visible
                Determines if an miniature box plot is drawn
                inside the violins.
            width
                Sets the width of the inner box plots relative
                to the violins' width. For example, with 1, the
                inner box plots are as wide as the violins.
""",
            **kwargs
        )



from ._visible import VisibleValidator
from ._valuesuffix import ValuesuffixValidator
from ._valueformat import ValueformatValidator
from ._uid import UidValidator
from ._textfont import TextfontValidator
from ._stream import StreamValidator
from ._showlegend import ShowlegendValidator
from ._selectedpoints import SelectedpointsValidator
from ._orientation import OrientationValidator
from ._opacity import OpacityValidator
from ._node import NodeValidator
from ._name import NameValidator
from ._link import LinkValidator
from ._legendgroup import LegendgroupValidator
from ._idssrc import IdssrcValidator
from ._ids import IdsValidator
from ._hoverlabel import HoverlabelValidator
from ._hoverinfosrc import HoverinfosrcValidator
from ._hoverinfo import HoverinfoValidator
from ._domain import DomainValidator
from ._customdatasrc import CustomdatasrcValidator
from ._customdata import CustomdataValidator
from ._arrangement import ArrangementValidator



from plotly.basedatatypes import BaseTraceHierarchyType
import copy


class Lightposition(BaseTraceHierarchyType):

    # x
    # -
    @property
    def x(self):
        """
        Numeric vector, representing the X coordinate for each vertex.
    
        The 'x' property is a number and may be specified as:
          - An int or float in the interval [-100000, 100000]

        Returns
        -------
        int|float
        """
        return self['x']

    @x.setter
    def x(self, val):
        self['x'] = val

    # y
    # -
    @property
    def y(self):
        """
        Numeric vector, representing the Y coordinate for each vertex.
    
        The 'y' property is a number and may be specified as:
          - An int or float in the interval [-100000, 100000]

        Returns
        -------
        int|float
        """
        return self['y']

    @y.setter
    def y(self, val):
        self['y'] = val

    # z
    # -
    @property
    def z(self):
        """
        Numeric vector, representing the Z coordinate for each vertex.
    
        The 'z' property is a number and may be specified as:
          - An int or float in the interval [-100000, 100000]

        Returns
        -------
        int|float
        """
        return self['z']

    @z.setter
    def z(self, val):
        self['z'] = val

    # property parent name
    # --------------------
    @property
    def _parent_path_str(self):
        return 'cone'

    # Self properties description
    # ---------------------------
    @property
    def _prop_descriptions(self):
        return """\
        x
            Numeric vector, representing the X coordinate for each
            vertex.
        y
            Numeric vector, representing the Y coordinate for each
            vertex.
        z
            Numeric vector, representing the Z coordinate for each
            vertex.
        """

    def __init__(self, arg=None, x=None, y=None, z=None, **kwargs):
        """
        Construct a new Lightposition object
        
        Parameters
        ----------
        arg
            dict of properties compatible with this constructor or
            an instance of plotly.graph_objs.cone.Lightposition
        x
            Numeric vector, representing the X coordinate for each
            vertex.
        y
            Numeric vector, representing the Y coordinate for each
            vertex.
        z
            Numeric vector, representing the Z coordinate for each
            vertex.

        Returns
        -------
        Lightposition
        """
        super(Lightposition, self).__init__('lightposition')

        # Validate arg
        # ------------
        if arg is None:
            arg = {}
        elif isinstance(arg, self.__class__):
            arg = arg.to_plotly_json()
        elif isinstance(arg, dict):
            arg = copy.copy(arg)
        else:
            raise ValueError(
                """\
The first argument to the plotly.graph_objs.cone.Lightposition 
constructor must be a dict or 
an instance of plotly.graph_objs.cone.Lightposition"""
            )

        # Handle skip_invalid
        # -------------------
        self._skip_invalid = kwargs.pop('skip_invalid', False)

        # Import validators
        # -----------------
        from plotly.validators.cone import (lightposition as v_lightposition)

        # Initialize validators
        # ---------------------
        self._validators['x'] = v_lightposition.XValidator()
        self._validators['y'] = v_lightposition.YValidator()
        self._validators['z'] = v_lightposition.ZValidator()

        # Populate data dict with properties
        # ----------------------------------
        _v = arg.pop('x', None)
        self['x'] = x if x is not None else _v
        _v = arg.pop('y', None)
        self['y'] = y if y is not None else _v
        _v = arg.pop('z', None)
        self['z'] = z if z is not None else _v

        # Process unknown kwargs
        # ----------------------
        self._process_kwargs(**dict(arg, **kwargs))

        # Reset skip_invalid
        # ------------------
        self._skip_invalid = False



import _plotly_utils.basevalidators


class ColoringValidator(_plotly_utils.basevalidators.EnumeratedValidator):

    def __init__(
        self,
        plotly_name='coloring',
        parent_name='histogram2dcontour.contours',
        **kwargs
    ):
        super(ColoringValidator, self).__init__(
            plotly_name=plotly_name,
            parent_name=parent_name,
            edit_type='calc',
            role='style',
            values=['fill', 'heatmap', 'lines', 'none'],
            **kwargs
        )



import _plotly_utils.basevalidators


class PullValidator(_plotly_utils.basevalidators.NumberValidator):

    def __init__(self, plotly_name='pull', parent_name='pie', **kwargs):
        super(PullValidator, self).__init__(
            plotly_name=plotly_name,
            parent_name=parent_name,
            array_ok=True,
            edit_type='calc',
            max=1,
            min=0,
            role='style',
            **kwargs
        )



import _plotly_utils.basevalidators


class ScattercarpetValidator(_plotly_utils.basevalidators.CompoundValidator):

    def __init__(self, plotly_name='scattercarpet', parent_name='', **kwargs):
        super(ScattercarpetValidator, self).__init__(
            plotly_name=plotly_name,
            parent_name=parent_name,
            data_class_str='Scattercarpet',
            data_docs="""
            a
                Sets the quantity of component `a` in each data
                point. If `a`, `b`, and `c` are all provided,
                they need not be normalized, only the relative
                values matter. If only two arrays are provided
                they must be normalized to match
                `ternary<i>.sum`.
            asrc
                Sets the source reference on plot.ly for  a .
            b
                Sets the quantity of component `a` in each data
                point. If `a`, `b`, and `c` are all provided,
                they need not be normalized, only the relative
                values matter. If only two arrays are provided
                they must be normalized to match
                `ternary<i>.sum`.
            bsrc
                Sets the source reference on plot.ly for  b .
            carpet
                An identifier for this carpet, so that
                `scattercarpet` and `scattercontour` traces can
                specify a carpet plot on which they lie
            connectgaps
                Determines whether or not gaps (i.e. {nan} or
                missing values) in the provided data arrays are
                connected.
            customdata
                Assigns extra data each datum. This may be
                useful when listening to hover, click and
                selection events. Note that, "scatter" traces
                also appends customdata items in the markers
                DOM elements
            customdatasrc
                Sets the source reference on plot.ly for
                customdata .
            fill
                Sets the area to fill with a solid color. Use
                with `fillcolor` if not "none". scatterternary
                has a subset of the options available to
                scatter. "toself" connects the endpoints of the
                trace (or each segment of the trace if it has
                gaps) into a closed shape. "tonext" fills the
                space between two traces if one completely
                encloses the other (eg consecutive contour
                lines), and behaves like "toself" if there is
                no trace before it. "tonext" should not be used
                if one trace does not enclose the other.
            fillcolor
                Sets the fill color. Defaults to a half-
                transparent variant of the line color, marker
                color, or marker line color, whichever is
                available.
            hoverinfo
                Determines which trace information appear on
                hover. If `none` or `skip` are set, no
                information is displayed upon hovering. But, if
                `none` is set, click and hover events are still
                fired.
            hoverinfosrc
                Sets the source reference on plot.ly for
                hoverinfo .
            hoverlabel
                plotly.graph_objs.scattercarpet.Hoverlabel
                instance or dict with compatible properties
            hoveron
                Do the hover effects highlight individual
                points (markers or line points) or do they
                highlight filled regions? If the fill is
                "toself" or "tonext" and there are no markers
                or text, then the default is "fills", otherwise
                it is "points".
            ids
                Assigns id labels to each datum. These ids for
                object constancy of data points during
                animation. Should be an array of strings, not
                numbers or any other type.
            idssrc
                Sets the source reference on plot.ly for  ids .
            legendgroup
                Sets the legend group for this trace. Traces
                part of the same legend group hide/show at the
                same time when toggling legend items.
            line
                plotly.graph_objs.scattercarpet.Line instance
                or dict with compatible properties
            marker
                plotly.graph_objs.scattercarpet.Marker instance
                or dict with compatible properties
            mode
                Determines the drawing mode for this scatter
                trace. If the provided `mode` includes "text"
                then the `text` elements appear at the
                coordinates. Otherwise, the `text` elements
                appear on hover. If there are less than 20
                points, then the default is "lines+markers".
                Otherwise, "lines".
            name
                Sets the trace name. The trace name appear as
                the legend item and on hover.
            opacity
                Sets the opacity of the trace.
            selected
                plotly.graph_objs.scattercarpet.Selected
                instance or dict with compatible properties
            selectedpoints
                Array containing integer indices of selected
                points. Has an effect only for traces that
                support selections. Note that an empty array
                means an empty selection where the `unselected`
                are turned on for all points, whereas, any
                other non-array values means no selection all
                where the `selected` and `unselected` styles
                have no effect.
            showlegend
                Determines whether or not an item corresponding
                to this trace is shown in the legend.
            stream
                plotly.graph_objs.scattercarpet.Stream instance
                or dict with compatible properties
            text
                Sets text elements associated with each (a,b,c)
                point. If a single string, the same string
                appears over all the data points. If an array
                of strings, the items are mapped in order to
                the the data points in (a,b,c).
            textfont
                Sets the text font.
            textposition
                Sets the positions of the `text` elements with
                respects to the (x,y) coordinates.
            textpositionsrc
                Sets the source reference on plot.ly for
                textposition .
            textsrc
                Sets the source reference on plot.ly for  text
                .
            uid

            unselected
                plotly.graph_objs.scattercarpet.Unselected
                instance or dict with compatible properties
            visible
                Determines whether or not this trace is
                visible. If "legendonly", the trace is not
                drawn, but can appear as a legend item
                (provided that the legend itself is visible).
            xaxis
                Sets a reference between this trace's x
                coordinates and a 2D cartesian x axis. If "x"
                (the default value), the x coordinates refer to
                `layout.xaxis`. If "x2", the x coordinates
                refer to `layout.xaxis2`, and so on.
            yaxis
                Sets a reference between this trace's y
                coordinates and a 2D cartesian y axis. If "y"
                (the default value), the y coordinates refer to
                `layout.yaxis`. If "y2", the y coordinates
                refer to `layout.yaxis2`, and so on.
""",
            **kwargs
        )



from ._yanchor import YanchorValidator
from ._y import YValidator
from ._xanchor import XanchorValidator
from ._x import XValidator
from ._visible import VisibleValidator
from ._transition import TransitionValidator
from ._tickwidth import TickwidthValidator
from ._ticklen import TicklenValidator
from ._tickcolor import TickcolorValidator
from ._templateitemname import TemplateitemnameValidator
from ._steps import StepsValidator
from ._pad import PadValidator
from ._name import NameValidator
from ._minorticklen import MinorticklenValidator
from ._lenmode import LenmodeValidator
from ._len import LenValidator
from ._font import FontValidator
from ._currentvalue import CurrentvalueValidator
from ._borderwidth import BorderwidthValidator
from ._bordercolor import BordercolorValidator
from ._bgcolor import BgcolorValidator
from ._activebgcolor import ActivebgcolorValidator
from ._active import ActiveValidator



import _plotly_utils.basevalidators


class SymbolValidator(_plotly_utils.basevalidators.CompoundValidator):

    def __init__(
        self,
        plotly_name='symbol',
        parent_name='layout.mapbox.layer',
        **kwargs
    ):
        super(SymbolValidator, self).__init__(
            plotly_name=plotly_name,
            parent_name=parent_name,
            data_class_str='Symbol',
            data_docs="""
            icon
                Sets the symbol icon image. Full list:
                https://www.mapbox.com/maki-icons/
            iconsize
                Sets the symbol icon size. Has an effect only
                when `type` is set to "symbol".
            text
                Sets the symbol text.
            textfont
                Sets the icon text font. Has an effect only
                when `type` is set to "symbol".
            textposition
                Sets the positions of the `text` elements with
                respects to the (x,y) coordinates.
""",
            **kwargs
        )



import _plotly_utils.basevalidators


class ContoursValidator(_plotly_utils.basevalidators.CompoundValidator):

    def __init__(
        self, plotly_name='contours', parent_name='contour', **kwargs
    ):
        super(ContoursValidator, self).__init__(
            plotly_name=plotly_name,
            parent_name=parent_name,
            data_class_str='Contours',
            data_docs="""
            coloring
                Determines the coloring method showing the
                contour values. If "fill", coloring is done
                evenly between each contour level If "heatmap",
                a heatmap gradient coloring is applied between
                each contour level. If "lines", coloring is
                done on the contour lines. If "none", no
                coloring is applied on this trace.
            end
                Sets the end contour level value. Must be more
                than `contours.start`
            labelfont
                Sets the font used for labeling the contour
                levels. The default color comes from the lines,
                if shown. The default family and size come from
                `layout.font`.
            labelformat
                Sets the contour label formatting rule using d3
                formatting mini-language which is very similar
                to Python, see: https://github.com/d3/d3-format
                /blob/master/README.md#locale_format.
            operation
                Sets the constraint operation. "=" keeps
                regions equal to `value` "<" and "<=" keep
                regions less than `value` ">" and ">=" keep
                regions greater than `value` "[]", "()", "[)",
                and "(]" keep regions inside `value[0]` to
                `value[1]` "][", ")(", "](", ")[" keep regions
                outside `value[0]` to value[1]` Open vs. closed
                intervals make no difference to constraint
                display, but all versions are allowed for
                consistency with filter transforms.
            showlabels
                Determines whether to label the contour lines
                with their values.
            showlines
                Determines whether or not the contour lines are
                drawn. Has an effect only if
                `contours.coloring` is set to "fill".
            size
                Sets the step between each contour level. Must
                be positive.
            start
                Sets the starting contour level value. Must be
                less than `contours.end`
            type
                If `levels`, the data is represented as a
                contour plot with multiple levels displayed. If
                `constraint`, the data is represented as
                constraints with the invalid region shaded as
                specified by the `operation` and `value`
                parameters.
            value
                Sets the value or values of the constraint
                boundary. When `operation` is set to one of the
                comparison values (=,<,>=,>,<=) "value" is
                expected to be a number. When `operation` is
                set to one of the interval values
                ([],(),[),(],][,)(,](,)[) "value" is expected
                to be an array of two numbers where the first
                is the lower bound and the second is the upper
                bound.
""",
            **kwargs
        )



import _plotly_utils.basevalidators


class DecreasingValidator(_plotly_utils.basevalidators.CompoundValidator):

    def __init__(
        self, plotly_name='decreasing', parent_name='candlestick', **kwargs
    ):
        super(DecreasingValidator, self).__init__(
            plotly_name=plotly_name,
            parent_name=parent_name,
            data_class_str='Decreasing',
            data_docs="""
            fillcolor
                Sets the fill color. Defaults to a half-
                transparent variant of the line color, marker
                color, or marker line color, whichever is
                available.
            line
                plotly.graph_objs.candlestick.decreasing.Line
                instance or dict with compatible properties
""",
            **kwargs
        )



import _plotly_utils.basevalidators


class DiagonalValidator(_plotly_utils.basevalidators.CompoundValidator):

    def __init__(self, plotly_name='diagonal', parent_name='splom', **kwargs):
        super(DiagonalValidator, self).__init__(
            plotly_name=plotly_name,
            parent_name=parent_name,
            data_class_str='Diagonal',
            data_docs="""
            visible
                Determines whether or not subplots on the
                diagonal are displayed.
""",
            **kwargs
        )



import _plotly_utils.basevalidators


class CalendarValidator(_plotly_utils.basevalidators.EnumeratedValidator):

    def __init__(
        self,
        plotly_name='calendar',
        parent_name='layout.scene.zaxis',
        **kwargs
    ):
        super(CalendarValidator, self).__init__(
            plotly_name=plotly_name,
            parent_name=parent_name,
            edit_type='calc',
            role='info',
            values=[
                'gregorian', 'chinese', 'coptic', 'discworld', 'ethiopian',
                'hebrew', 'islamic', 'julian', 'mayan', 'nanakshahi', 'nepali',
                'persian', 'jalali', 'taiwan', 'thai', 'ummalqura'
            ],
            **kwargs
        )



import _plotly_utils.basevalidators


class AngularAxisValidator(_plotly_utils.basevalidators.CompoundValidator):

    def __init__(
        self, plotly_name='angularaxis', parent_name='layout', **kwargs
    ):
        super(AngularAxisValidator, self).__init__(
            plotly_name=plotly_name,
            parent_name=parent_name,
            data_class_str='AngularAxis',
            data_docs="""
            domain
                Polar chart subplots are not supported yet.
                This key has currently no effect.
            endpadding

            range
                Defines the start and end point of this angular
                axis.
            showline
                Determines whether or not the line bounding
                this angular axis will be shown on the figure.
            showticklabels
                Determines whether or not the angular axis
                ticks will feature tick labels.
            tickcolor
                Sets the color of the tick lines on this
                angular axis.
            ticklen
                Sets the length of the tick lines on this
                angular axis.
            tickorientation
                Sets the orientation (from the paper
                perspective) of the angular axis tick labels.
            ticksuffix
                Sets the length of the tick lines on this
                angular axis.
            visible
                Determines whether or not this axis will be
                visible.
""",
            **kwargs
        )



import _plotly_utils.basevalidators


class YAxisValidator(_plotly_utils.basevalidators.SubplotidValidator):

    def __init__(
        self, plotly_name='yaxis', parent_name='contourcarpet', **kwargs
    ):
        super(YAxisValidator, self).__init__(
            plotly_name=plotly_name,
            parent_name=parent_name,
            dflt='y',
            edit_type='calc+clearAxisTypes',
            role='info',
            **kwargs
        )



import _plotly_utils.basevalidators


class StartsValidator(_plotly_utils.basevalidators.CompoundValidator):

    def __init__(
        self, plotly_name='starts', parent_name='streamtube', **kwargs
    ):
        super(StartsValidator, self).__init__(
            plotly_name=plotly_name,
            parent_name=parent_name,
            data_class_str='Starts',
            data_docs="""
            x
                Sets the x components of the starting position
                of the streamtubes
            xsrc
                Sets the source reference on plot.ly for  x .
            y
                Sets the y components of the starting position
                of the streamtubes
            ysrc
                Sets the source reference on plot.ly for  y .
            z
                Sets the z components of the starting position
                of the streamtubes
            zsrc
                Sets the source reference on plot.ly for  z .
""",
            **kwargs
        )
